<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00001000ItemLancamentoPropostaFactory
{
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Exclui um item da tabela de agendamentos pendentes caso ele seja excluido na proposta
    **/
    public function excluiItemAgendPendente(array $params){
        $sql = "DELETE FROM FCT_AGENDPENDENTE
                WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                    AND (NRITEMORCAMENTO = :NRITEMORCAMENTO
                        OR :NRITEMORCAMENTO IS NULL)
                    AND NRORG            = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Atualiza a quantidade de um item da tabela de agendamentos pendentes 
     * caso ele seja alterado na proposta
    **/
    public function atualizaQuantidadeItensAgendPendente(array $params){
        $sql = "UPDATE FCT_AGENDPENDENTE
                SET QTPENDENCIA         = :QTPENDENCIA
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                    AND NRITEMORCAMENTO = :NRITEMORCAMENTO
                    AND NRORG           = :NRORG ";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * retorna a quantidade de elementos distintos de uma árvore de elementos
    **/
    public function getCountElementos(array $params){
        $sql = "SELECT COUNT(IDELEMENTO) QTELEMENTOS, 
                  SUM(NVL(VRTOTSAJUSTE, 0)) AS VRTOTSAJUSTE
                FROM (SELECT DISTINCT IDELEMENTO, VRTOTSAJUSTE
                  FROM FCT_TRABSERVORCAMENTO
                  WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                     AND (IDELEMENTOPAI  = :IDELEMENTOPAI
                          OR IDELEMENTOPAI LIKE :IDELEMENTOPAI||'%')
                     AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                     AND CDSERVORCA      = :CDSERVORCA
                     AND NRORG           = :NRORG
                     AND IDESCOLHIDO     = 'S'
                     AND IDATIVO         = 'S'
                     AND (CDTPMEDIDASERV  = :CDTPMEDIDASERV OR CDTPMEDIDASERV IS NULL))";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $quantidade = $std->fetchAll();
        return $quantidade[0];
    }

    /**
     * Retorna os elementos de cada orcamento com o determinado servico
     * e as infomações do valor da medida para serem alterados na proposta
    **/
    public function getElementosComposicao(array $params){
        $sql = "SELECT IDELEMENTO,
                  IDELEMENTOPAI,
                  QTMEDIDASERV
                FROM FCT_SERVICOORCA SERV
                LEFT JOIN FCT_RELACSERVICO SERVRELAC
                  ON SERV.CDSERVORCA = SERVRELAC.CDSERVORCA
                  AND SERV.NRORG     = SERVRELAC.NRORG
                WHERE SERVRELAC.CDSERVORCARELAC = :CDSERVORCA
                CONNECT BY PRIOR IDELEMENTO     = IDELEMENTOPAI
                START WITH IDELEMENTO           = :IDELEMENTO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Altera o valor unitário e o valor sem desconto do elemento 
    **/
    public function atualizarPrecoElementoItem(array $params){
        $sql = "UPDATE FCT_TRABSERVORCAMENTO
                SET QTMEDSERVORCA    = ROUND(:QTMEDSERVORCA, 2),
                    VRUNITSERVORCA   = ROUND(:VRUNITSERVORCA, 2),
                    QTMEDIDACALC     = ROUND(:QTMEDSERVORCA, 2),
                    QTMEDIDAVARIAVEL = ROUND(:QTMEDSERVORCA, 2),
                    VRTOTSAJUSTE     = ROUND(:VRTOTSAJUSTE, 2),
                    VRTOTSERVORCA    = ROUND(:VRTOTSERVORCA, 2)
                WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                    AND NRORG          = :NRORG
                    AND CDSERVORCA     = :CDSERVORCA
                    AND (IDELEMENTOPAI = :IDELEMENTOPAI
                        AND (IDELEMENTO = 'USUARIO' OR IDELEMENTO = :IDELEMENTO))
                    AND CDTPMEDIDASERV = :CDTPMEDIDASERV";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    /**
     * Atualiza o vaor do item da proposta
    **/
    public function atualizarValorItemProposta($params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRTOTALITEMORCA          = ROUND(:VRTOTALITEMORCA, 2),
                    VRUNITITEMORCATBLPRECO   = ROUND(:VRUNITITEMORCATBLPRECO, 2),
                    QTMEDIDAORCA             = ROUND(:QTMEDIDAORCA, 2),
                    VRITEMSEMDESC            = ROUND(:VRITEMSEMDESC, 2)
                WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                    AND NRORG           = :NRORG
                    AND NRITEMORCAMENTO = :NRITEMORCAMENTO
                    AND CDSERVORCA      = :CDSERVORCA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna o somatorio do elementos de um determinado servico
    **/
    public function getValorTotalItensTipoElemento(array $params){
        $sql = "SELECT SUM(VRTOTALITEMORCA) AS VRTOTALITEMORCA
                FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND CDSERVORCA   = :CDSERVORCA
                    AND NRORG        = :NRORG
                    AND IDELEMENTO IS NOT NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0]["VRTOTALITEMORCA"];
    }
    /**
     * Altera o valor total do item após as alterações no valor de um elemento
    **/
    public function replicaAteracaoParaItemPai(array $params, $cdoperador){
        $vrtotservorca = $this->getValorTotalItensTipoElemento($params);
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET VRUNITITEMORCA  = ROUND($vrtotservorca, 2),
                    VRTOTALITEMORCA = ROUND($vrtotservorca, 2),
                    VRITEMSEMDESC   = ROUND($vrtotservorca, 2),
                    CDOPERULTATU    = $cdoperador
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND CDSERVORCA   = :CDSERVORCA
                    AND NRORG        = :NRORG
                    AND IDELEMENTO IS NULL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    /**
     * Retorna todas as informações referentes ao valor do item da proposta 
    **/
    public function getInfoValoresItensOrcamento(array $params){
        $sql = "SELECT DISTINCT VRUNITITEMORCA,
                        IDMULTIPVARIACAO,
                        IDMEDIDAFIXA,
                        NRITEMORCAMENTO,
                        CDSERVORCARELAC,
                        VRPRECO,
                        IDELEMENTO,
                        VRTOTALITEMORCA,
                        QTMEDIDAORCA,
                        VRPERCDESCORCA,
                        IDTPDESCORCA,
                        NRTIPOORCAMENTO,
                        CDTPMEDIDASERV,
                        CDTPMEDIDAVARIAVEL,
                        VRUNITITEMORCATBLPRECO,
                        SUM(QTMEDIDASERV) AS QTMEDIDASERV
                    FROM (SELECT ITEM.VRUNITITEMORCA,
                            NVL(VAR.IDMULTIPVARIACAO, 'N') AS IDMULTIPVARIACAO,
                            RELACSERV.IDMEDIDAFIXA,
                            ITEM.NRITEMORCAMENTO,
                            RELACSERV.CDSERVORCARELAC,
                            NVL(VAR.VRPRECOVARIACAO, ITVAR.VRPRECO) AS VRPRECO,
                            RELACSERV.QTMEDIDASERV,
                            ITEM.IDELEMENTO,
                            ITEM.VRTOTALITEMORCA,
                            ITEM.QTMEDIDAORCA,
                            ITEM.VRPERCDESCORCA,
                            ITEM.IDTPDESCORCA,
                            ITEM.NRTIPOORCAMENTO,
                            ITEM.CDTPMEDIDASERV,
                            ITEM.CDTPMEDIDAVARIAVEL,
                            ITEM.VRUNITITEMORCATBLPRECO
                        FROM FCT_ITEMORCAMENTO ITEM
                        INNER JOIN FCT_RELACSERVICO RELACSERV
                            ON ITEM.CDSERVORCA      = RELACSERV.CDSERVORCARELAC
                            AND ITEM.NRORG          = RELACSERV.NRORG
                            AND ITEM.CDTPMEDIDASERV = RELACSERV.CDTPMEDIDASERV
                        INNER JOIN FCT_SERVICOORCA SERVICO
                            ON RELACSERV.CDSERVORCA = SERVICO.CDSERVORCA
                            AND RELACSERV.NRORG = SERVICO.NRORG
                        LEFT JOIN FCT_RELACMEDIDAPROPOSTA MEDIDA
                            ON ITEM.IDELEMENTO      = MEDIDA.IDELEMENTORAIZ
                            AND ITEM.NRORG          = MEDIDA.NRORG
                            AND ITEM.NRSEQORCAMENTO = MEDIDA.NRSEQORCAMENTO
                        INNER JOIN FCT_ITPRECBASESERV ITVAR
                            ON RELACSERV.CDSERVORCARELAC = ITVAR.CDSERVORCA
                            AND RELACSERV.NRORG          = ITVAR.NRORG
                        LEFT JOIN FCT_VARIACAOPRECBASESERV VAR
                            ON  ITVAR.NRSEQITPRECBSERV = VAR.NRSEQITPRECBSERV
                            AND ITVAR.NRORG            = VAR.NRORG
                            AND ITEM.CDTPMEDIDAVARIAVEL = VAR.CDTPMEDIDAVARIAVEL
                        WHERE ITEM.NRSEQORCAMENTO = :NRSEQORCAMENTO
                            AND ITEM.NRORG          = :NRORG
                            AND (VAR.NRVARIACAOPRECBASESERV IS NULL
                                OR MEDIDA.QTMEDIDAVARIAVEL BETWEEN VAR.QTMEDIDAINIVARIACAO AND QTMEDIDAFIMVARIACAO
                                AND VAR.DTFIMVALPBSERV IS NULL AND TRUNC(SYSDATE) >= TRUNC(VAR.DTINIPBSERV)
                                        OR TRUNC(SYSDATE) BETWEEN TRUNC(VAR.DTINIPBSERV) AND TRUNC(VAR.DTFIMVALPBSERV))
                            AND ITEM.IDELEMENTO IS NOT NULL
                            AND SERVICO.IDELEMENTO IN (SELECT IDELEMENTO 
                                                        FROM FCT_TRABSERVORCAMENTO
                                                        WHERE NRSEQORCAMENTO = ITEM.NRSEQORCAMENTO
                                                        AND IDESCOLHIDO    = 'S'
                                                        AND CDSERVORCA     = ITEM.CDSERVORCA
                                                        AND NRORG          = ITEM.NRORG)
                            AND RELACSERV.CDSERVORCA IN(SELECT CDSERVORCA
                                                        FROM FCT_SERVICOORCA
                                                        CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                                                        START WITH IDELEMENTO = ITEM.IDELEMENTO))
                        GROUP BY VRUNITITEMORCA,
                            IDMULTIPVARIACAO,
                            IDMEDIDAFIXA,
                            NRITEMORCAMENTO,
                            CDSERVORCARELAC,
                            VRPRECO,
                            IDELEMENTO,
                            VRTOTALITEMORCA,
                            QTMEDIDAORCA,
                            VRPERCDESCORCA,
                            IDTPDESCORCA,
                            NRTIPOORCAMENTO,
                            CDTPMEDIDASERV,
                            CDTPMEDIDAVARIAVEL,
                            VRUNITITEMORCATBLPRECO
                        ORDER BY CDSERVORCARELAC";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Atualiza o nome contatual dos itens
    **/
    public function salvarNomeContratual(array $params){
        $sql = "UPDATE FCT_ITEMORCAMENTO
                SET NMCONTRATUALITEMORCA = :NMCONTRATUALITEMORCA,
                    CDOPERULTATU         = :CDOPERADOR
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND IDELEMENTO   = :IDELEMENTO
                    AND NRORG        = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * retorna a quantidade de composições de serviço inativas na proposta
    **/
    public function getTotalComposicoesInativas(array $params){
        $sql = "SELECT COUNT(*) AS QTCOMPINATIVA
                FROM FCT_COMPSERVORCAMENTO ORCACOMP
                INNER JOIN FCT_COMPSERVICO COMP
                  ON ORCACOMP.NRCOMPSERVICO = COMP.NRCOMPSERVICO
                  AND ORCACOMP.NRORG        = COMP.NRORG
                WHERE ORCACOMP.NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND ORCACOMP.NRORG          = :NRORG
                  AND ORCACOMP.IDESCOLHIDO    = 'S'
                  AND COMP.IDATIVO            = 'N'";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll()[0]["QTCOMPINATIVA"];
    }
    
    /**
     * Retorna os elementos que necessitam da criação de um agrupador
    **/
    public function getElementosComMaisDeUmaMedida(array $params){
        $sql = "SELECT SUM(VRTOTALITEMORCA) AS VRTOTALITEMORCA, 
                  NRITEMORCAMENTOPAI,
                  CDSERVORCA,
                  IDELEMENTO,
                  NRITEMORCAMENTOPAI || '_' || IDELEMENTO AS IDARVORE
                FROM FCT_ITEMORCAMENTO
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND IDELEMENTO IN((SELECT IDELEMENTORAIZ 
                                     FROM (SELECT NRSEQORCAMENTO, 
                                            IDELEMENTORAIZ, 
                                            COUNT(CDTPMEDIDAVARIAVEL) AS QUANTIDADE
                                          FROM FCT_RELACMEDIDAPROPOSTA
                                          GROUP BY NRSEQORCAMENTO, IDELEMENTORAIZ)
                                    WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                        AND NRORG        = :NRORG
                                        AND QUANTIDADE > 1))
                GROUP BY NRITEMORCAMENTOPAI,
                  CDSERVORCA,
                  IDELEMENTO
                HAVING COUNT(*) > 1";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function verificarSeExisteDescontoNegociacaoProposta(array $params) {
        $sql = "SELECT NVL(VRPERCDESCITEM, 0) AS VRPERCDESCITEM
                  FROM FCT_NEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getSomaPercentualDescontoItensNegociacaoProposta(array $params) {
        $sql = "SELECT NVL(SUM(VRPERCDESCITEM), 0) AS VRPERCDESCITEM
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getServicosNegociacaoProposta(array $params) {
        $sql = "SELECT DISTINCT CDSERVORCA
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacaoItemProposta(array $params) {
        $sql = "SELECT NRNEGOCIACAOPROPOSTA
                  FROM FCT_ITNEGOCIACAOPROPOSTA
                 WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                   AND NRORG           = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoItemPropostaPai(array $params) {
        $sql = "SELECT NRITEMORCAMENTOPAI
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                   AND NRORG           = :NRORG";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getParametrosFaturamentoParaItemNegociacaoProposta($params) {
        $sql = "SELECT PFESC.NRTIPOORCAMENTO,
                       ITORC.NRITEMORCAMENTO,
                       ITORC.NRITEMORCAMENTOPAI,
                       ITORC.CDSERVORCA,
                       ITORC.VRTOTALITEMORCA,
                       NVL(PFIT.IDTPFATURA, PFESC.IDTPFATURA) AS IDTPFATURA,
                       NVL(PFIT.CDTIPOOPERFATURA, PFESC.CDTIPOOPERFATURA) AS CDTIPOOPERFATURA,
                       NVL(PFIT.CDSERIENF, PFESC.CDSERIENF) AS CDSERIENF,
                       NVL(PFIT.CDEMPRESA, PFESC.CDEMPRESA) AS CDEMPRESA,
                       NVL(PFIT.CDINSCESTA, PFESC.CDINSCESTA) AS CDINSCESTA,
                       NVL(PFIT.CDINSCMUNI, PFESC.CDINSCMUNI) AS CDINSCMUNI,
                       NVL(ITORC.CDPRODUTO, NVL(PFIT.CDPRODFATURA, NVL(PFESC.CDPRODFATURA, SERV.CDPRODFATSERV))) AS CDPRODFATURA,
                       NVL(PFIT.CDFILIALFATURA, PFESC.CDFILIALFATURA) AS CDFILIALFATURA,
                       NVL(PFIT.NRPARCFATURA, NVL(PFESC.NRPARCFATURA, ORC.NRPARCNEGOCIO)) AS NRPARCFATURA,
                       NVL(PFIT.QTPARCELASFATU, PFESC.QTPARCELASFATU) AS QTPARCELASFATU,
                       NVL(PFESC.QTPARCELASFATU, 1) AS QTPARCELASFATUESC,
                       NVL(PFIT.QTDIASAPURACAO, PFESC.QTDIASAPURACAO) AS QTDIASAPURACAO,
                       NVL(PFIT.NRDIACALCVENC, PFESC.NRDIACALCVENC) AS NRDIACALCVENC,
                       NVL(PFIT.QTDIASFATUVENC, PFESC.QTDIASFATUVENC) AS QTDIASFATUVENC,
                       NVL(PFIT.IDTPCALCVENC, PFESC.IDTPCALCVENC) AS IDTPCALCVENC,
                       NVL(PFIT.IDMESCALCVENC, PFESC.IDMESCALCVENC) AS IDMESCALCVENC,
                       NVL(ITESC.IDPREVCOBRFUT, 'N') AS IDPREVCOBRFUT
                  FROM FCT_CAPAPARAMFATESC PFESC
                  LEFT JOIN FCT_ITESCOPOPROPOSTA ITESC
                    ON PFESC.NRTIPOORCAMENTO = ITESC.NRTIPOORCAMENTO
                   AND PFESC.NRORG           = ITESC.NRORG
                   AND ITESC.CDSERVORCA      = :CDSERVORCA
                  LEFT JOIN FCT_PARAMFATITESC PFIT
                    ON ITESC.NRTIPOORCAMENTO    = PFIT.NRTIPOORCAMENTO
                   AND ITESC.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                   AND ITESC.NRORG              = PFIT.NRORG
                 INNER JOIN FCT_ORCAMENTO ORC
                    ON PFESC.NRORG        = ORC.NRORG
                   AND ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                 INNER JOIN FCT_ITEMORCAMENTO ITORC
                    ON ORC.NRSEQORCAMENTO    = ITORC.NRSEQORCAMENTO
                   AND ORC.NRORG             = ITORC.NRORG
                   AND ITORC.NRITEMORCAMENTO = :NRITEMORCAMENTO
                  LEFT JOIN FCT_SERVICOORCA SERV
                    ON ITORC.CDSERVORCA = SERV.CDSERVORCA
                   AND ITORC.NRORG      = SERV.NRORG
                 WHERE PFESC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND PFESC.NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getNegociacaoPropostaPorParametrosFaturamento(array $params) {
        $sql = "SELECT NRNEGOCIACAOPROPOSTA
                  FROM FCT_NEGOCIACAOPROPOSTA
                 WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                   AND NRORG            = :NRORG
                   AND NRTIPOORCAMENTO  = :NRTIPOORCAMENTO
                   AND IDTPFATURA       = :IDTPFATURA
                   AND CDFILIALFATURA   = :CDFILIALFATURA
                   AND NRPARCFATURA     = :NRPARCFATURA
                   AND CDTIPOOPERFATURA = :CDTIPOOPERFATURA
                   AND CDSERIENF        = :CDSERIENF
                   AND CDEMPRESA        = :CDEMPRESA
                   AND CDINSCESTA       = :CDINSCESTA
                   AND CDINSCMUNI       = :CDINSCMUNI
                   AND IDPREVCOBRFUT    =  'N'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function excluirItemProposta(array $params) {
        $sql = "DELETE FROM FCT_ITEMORCAMENTO
                 WHERE NRITEMORCAMENTO = :NRITEMORCAMENTO
                   AND NRORG           = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    /**
     * Retorna o valor total de um item da negociação 
    **/
    public function getValorItemNegociacaoDeValores(array $params) {
        $sql = "SELECT VRTOTITEMCOMDESC, VRDESCITEM, CDSERVORCA
                FROM FCT_ITNEGOCIACAOPROPOSTA 
                WHERE (NRITEMORCAMENTO = :NRITEMORCAMENTO OR :NRITEMORCAMENTO IS NULL)
                   AND NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND NRORG                = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Busca o valor do serviço na tabela de preço
    **/
    public function getValorUnitarioTblPrecoItemProposta(array $params) {        
        $sql = "SELECT NVL(VARPREC.VRPRECOVARIACAO, ITPREC.VRPRECO) AS VRPRECO
                  FROM FCT_ITPRECBASESERV ITPREC
                  LEFT JOIN FCT_VARIACAOPRECBASESERV VARPREC
                    ON ITPREC.NRSEQITPRECBSERV    = VARPREC.NRSEQITPRECBSERV
                   AND ITPREC.NRORG               = VARPREC.NRORG
                   AND VARPREC.CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                   AND :QTMEDIDASERV BETWEEN VARPREC.QTMEDIDAINIVARIACAO AND VARPREC.QTMEDIDAFIMVARIACAO
                 INNER JOIN FCT_TIPOORCAMENTO ESC
                    ON ITPREC.CDPRECBASESERV = ESC.CDPRECBASESERV
                   AND ITPREC.NRORG          = ESC.NRORG
                 WHERE ESC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND ITPREC.CDSERVORCA   = :CDSERVORCA
                   AND ITPREC.NRORG        = :NRORG";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Atualiza o valor de um item na tabela de rateio
    **/
    public function atualizarValorRateioItens(array $params) {
        $sql = "UPDATE FCT_RELACESTRUTURAPROP
                SET VRITEMRATEIO = ROUND(:VRTOTALITEM * (VRPERCRATEIO/100), 2),
                    CDOPERULTATU = :CDOPERULTATU,
                    DTULTATU     = SYSDATE
                WHERE NRITEMORCAMENTO  = :NRITEMORCAMENTO
                    AND NRSEQORCAMENTO = :NRSEQORCAMENTO 
                    AND NRORG          = :NRORG";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

    public function getMedidaVariavelEscopo(array $params) {
        $sql = "SELECT *
                  FROM (
                      SELECT NVL(PFIT.CDTPMEDIDAVARIAVEL, PF.CDTPMEDIDAVARIAVEL) AS CDTPMEDIDAVARIAVEL,
                             MEDIDA.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL
                        FROM FCT_CAPAPARAMFATESC PF
                       INNER JOIN FCT_TIPOORCAMENTO ES
                          ON PF.NRTIPOORCAMENTO = ES.NRTIPOORCAMENTO
                         AND PF.NRORG           = ES.NRORG
                        LEFT JOIN FCT_ITESCOPOPROPOSTA IT
                          ON PF.NRTIPOORCAMENTO = IT.NRTIPOORCAMENTO
                         AND PF.NRORG           = IT.NRORG
                         AND IT.CDSERVORCA      = :CDSERVORCA
                        LEFT JOIN FCT_PARAMFATITESC PFIT
                          ON IT.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                         AND IT.NRORG              = PFIT.NRORG
                        LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                          ON NVL(PFIT.CDTPMEDIDAVARIAVEL, PF.CDTPMEDIDAVARIAVEL) = MEDIDA.CDTPMEDIDASERV
                         AND PF.NRORG                                            = MEDIDA.NRORG
                       WHERE PF.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                         AND PF.NRORG             = :NRORG
                         AND ES.IDUTILIZAVARPRECO = 'S') TBL
                 WHERE CDTPMEDIDAVARIAVEL IS NOT NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function excluirMedidaVariavelServico(array $params) {
        $sql = "DELETE FROM FCT_RELACMEDIDAPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND CDSERVORCA     = :CDSERVORCA
                   AND NRORG          = :NRORG
                   AND IDELEMENTORAIZ IS NULL";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function getMedidasVariaveisPropostaSemItemRelacionado(array $params) {
        $sql = "SELECT NRRELACMEDIDAPROPOSTA,
                       CDTPMEDIDAVARIAVEL,
                       QTMEDIDAVARIAVEL,
                       CDSERVORCA
                  FROM FCT_RELACMEDIDAPROPOSTA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG
                   AND IDELEMENTORAIZ IS NULL
                   AND CDSERVORCA     NOT IN (SELECT NVL(CDSERVORCA, 0)
                                                FROM FCT_ITEMORCAMENTO
                                               WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                                                 AND NRORG          = :NRORG)";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function pegarPerfilDeAcessoDesseOperadorLogado(array $params){
        $sql = "SELECT PAP.NRPERFILACESSO, OP.NRPARCNEGOCIO FROM PERFACESSPARC PAP JOIN OPERADOR OP ON PAP.NRPARCNEGOCIO = OP.NRPARCNEGOCIO WHERE OP.CDOPERADOR = :CDOPERADOR";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Pega a quantidade de descontos dado por este operador neste contrato/proposta
    */
    public function pegarDadosCarenciaOperador (array $params){
        $sql = "SELECT DP.CDOPERULTATU, SUM(DP.QTMESESDESC) AS QTPARCELAS FROM FCT_DESCPERIODONEGOPROP DP JOIN OPERADOR OPER ON DP.CDOPERULTATU = OPER.CDOPERADOR WHERE DP.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA AND DP.CDOPERULTATU = :CDOPERADOR GROUP BY DP.CDOPERULTATU";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Pega a quantidade de descontos dado por este perfil de acesso neste contrato/proposta
    */
    public function pegarDadosCarenciaPerfil(array $params, $perfis){
        $sql = "SELECT PERF.NRPERFILACESSO, PERFILACESSO.NMPERFILACESSO, SUM(DP.QTMESESDESC) AS QTPARCELAS FROM FCT_DESCPERIODONEGOPROP DP JOIN OPERADOR OPER ON DP.CDOPERULTATU = OPER.CDOPERADOR JOIN PERFACESSPARC PERF ON OPER.NRPARCNEGOCIO = PERF.NRPARCNEGOCIO JOIN PERFILACESSO ON PERF.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO WHERE DP.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA AND PERF.NRPERFILACESSO IN (".$perfis.") GROUP BY PERF.NRPERFILACESSO, PERFILACESSO.NMPERFILACESSO";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    
    public function pegarDadosFaixaDescontoOperadorEPerfil(array $params, $perfis){
        $sql = "SELECT  'OPERADOR' AS IDENTIFICADOR, FCT_CARENCIAFAIXADESC.NRPARCELASMAX, FCT_CARENCIAFAIXADESC.VRPERCDESCDE, FCT_CARENCIAFAIXADESC.VRPERCDESCATE, C.NRPARCELASMAX AS MAXIMO, C.VRPERCMAXATE  FROM  FCT_CARENCIAFAIXADESC  JOIN  FCT_CARENCIAOPER C ON C.NRCARENCIAOPER = FCT_CARENCIAFAIXADESC.NRCARENCIAOPER  WHERE  C.NRPARCNEGOCIO = :NRPARCNEGOCIO  UNION ALL  SELECT  'PERFIL' AS IDENTIFICADOR, FCT_CARENCIAFAIXADESC.NRPARCELASMAX, FCT_CARENCIAFAIXADESC.VRPERCDESCDE, FCT_CARENCIAFAIXADESC.VRPERCDESCATE, C.NRPARCELASMAX AS MAXIMO, C.VRPERCMAXATE  FROM  FCT_CARENCIAFAIXADESC  JOIN  FCT_CARENCIAPERFIL C ON C.NRCARENCIAPERFIL = FCT_CARENCIAFAIXADESC.NRCARENCIAPERFIL  WHERE  FCT_CARENCIAFAIXADESC.NRCARENCIAPERFIL IN(SELECT NRCARENCIAPERFIL FROM FCT_CARENCIAPERFIL WHERE NRPERFILACESSO IN (".$perfis."))";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function pegarDadosCarenciaOperadorContrato (array $params){
        $sql = "SELECT DP.CDOPERULTATU, SUM(DP.QTMESESDESC) AS QTPARCELAS FROM FCT_DESCPERIODONEGOCONT DP JOIN OPERADOR OPER ON DP.CDOPERULTATU = OPER.CDOPERADOR WHERE DP.NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO AND DP.CDOPERULTATU = :CDOPERADOR GROUP BY DP.CDOPERULTATU";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function pegarDadosCarenciaPerfilContrato(array $params, $perfis){
        $sql = "SELECT PERF.NRPERFILACESSO, PERFILACESSO.NMPERFILACESSO, SUM(DP.QTMESESDESC) AS QTPARCELAS FROM FCT_DESCPERIODONEGOCONT DP JOIN OPERADOR OPER ON DP.CDOPERULTATU = OPER.CDOPERADOR JOIN PERFACESSPARC PERF ON OPER.NRPARCNEGOCIO = PERF.NRPARCNEGOCIO JOIN PERFILACESSO ON PERF.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO WHERE DP.NRNEGOCIACAOCONTRATO = :NRNEGOCIACAOCONTRATO AND PERF.NRPERFILACESSO IN (".$perfis.") GROUP BY PERF.NRPERFILACESSO, PERFILACESSO.NMPERFILACESSO";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    
    public function pegarDadosCarenciaMaximaOperador(array $params){
        $sql = "SELECT NRPARCELASMAX, VRPERCMAXATE FROM  FCT_CARENCIAOPER  WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO";
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function pegarDadosCarenciaMaximaPerfilAcesso($perfis){
        $sql = "SELECT  MAX(NRPARCELASMAX) AS NRPARCELASMAX, MAX(VRPERCMAXATE) AS VRPERCMAXATE FROM FCT_CARENCIAPERFIL WHERE NRPERFILACESSO IN (".$perfis.") ";
        $ps = $this->connection->prepare($sql);
        $ps->execute();
        return $ps->fetchAll();
    }
}
?>